#ifndef __SOCKET_H
#define __SOCKET_H


#include <unistd.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>


//
// TCP/IP socket
//
class Socket {

    private:
	int		sfd;
	unsigned short	sport;

    public:
	// constructors and destructor
	Socket(const Socket&);
	Socket(int = -1);
	~Socket();

	//
	// start a TCP server: Open --> Listen --> Accept
	// start a TCP client: Open --> Connect
	// stop a server/client: Close
	//
	// start a UDP socket: Open
	//
	bool Open(const char*, unsigned short = 0);
	bool Close();
	bool Listen(int = 1);
	bool Connect(const char*, unsigned short);
	Socket* Accept();

	// wait for data
	bool hasData(int = -1);

	// send and receive data
	int Send(const void*, int);
	int Recv(void*, int);
	int SendTo(const char*, unsigned short, const void*, int);
	int SendTo(struct sockaddr_in*, const void*, int);
	int RecvFrom(struct sockaddr_in*, void*, int);

	// information about this socket
	// (this is only valid if sfd >= 0)
	int sock() const		{ return sfd; }
	unsigned short port() const	{ return sport; }
};


#endif // __SOCKET_H
